/**This class contains methods for setting the load load set points of the generators in an 
 * unbalanced setting. This is the first and most primitive version of the optimization
 * algorithm.
 * 
 * @author Andreas Carlsen
 *
 */
public class simpleLoadSetPoints{
	private int nr_of_gen;
	private double[] w_max;
	private double[][] SFOC;
	private int[] low_limit;
	private int[] up_limit;
	private double demand;
	private int[] bin_var;
	
	/**Constructor that initializes the variables
	 * 
	 * @param w_max				Contains the maximum power for each generator
	 * @param SFOC				Contains the SFOC for each generator
	 * @param low_limit			Contains the low limit for each generator
	 * @param high_limit		Contains the high limit for each generator
	 */
	public simpleLoadSetPoints(double[] w_max, double[][] SFOC, int[] low_limit, int[] up_limit){
		this.nr_of_gen = w_max.length;
		this.w_max = w_max;
		this.SFOC = SFOC;
		this.low_limit = low_limit;
		this.up_limit = up_limit;
		this.bin_var = new int[this.nr_of_gen];
	}

	/**Returns the maximum power for each generator
	 * 
	 * @return					The maximum power for each generator (w_max)
	 */
	public double[] getW_max() {
		return w_max;
	}

	/**Sets the low limit for each generator
	 * 
	 * @param low_limit			Low limit of each generator
	 */
	public void setLow_limit(int[] low_limit) {
		this.low_limit = low_limit;
	}
	
	/**Sets the high limit for each generator
	 * 
	 * @param up_limit			High limit of each generator
	 */
	public void setUp_limit(int[] up_limit) {
		this.up_limit = up_limit;
	}

	/**Sets the demand
	 * 
	 * @param demand			Power demand
	 */
	public void setDemand(double demand) {
		this.demand = demand;
	}
	
	/**Sets the running bit for each generator
	 * 
	 * @param bin_var			Running bit for each generator
	 */
	public void setBin_var(int[] bin_var) {
		this.bin_var = bin_var;
	}
	
	/**Returns true if the power production is sufficient with the given loads.
	 * 
	 * @param load				Load of each generator
	 * @return 					True if the power production is sufficient, false if not.
	 */
	public boolean isNecessaryProduction(int[] load){
		double total_production = 0;
		for(int i=0; i<this.nr_of_gen;i++){
			total_production += (load[i]*this.w_max[i]*this.bin_var[i])/100;
		}
		return total_production >= this.demand;
	}

	/**Decides the number of active generators and runs the associated algorithm.
	 * 
	 * @return 					The optimal load set points.
	 */
	public int[] calculateOptimalLoad(){
		int nr_of_gens = 0;
		int[] active_gens = new int[4];
		for(int i=0; i<4; i++){
			if(this.bin_var[i]==1){
				active_gens[nr_of_gens] = i;
				nr_of_gens++;	
			}
		}
		if(nr_of_gens == 1)
			return  OptimalLoad1Gen(active_gens[0]);
		else if(nr_of_gens == 2)
			return  OptimalLoad2Gens(active_gens[0], active_gens[1]);
		else if(nr_of_gens == 3)
			return  OptimalLoad3Gens(active_gens[0], active_gens[1], active_gens[2]);
		else if(nr_of_gens == 4)
			return  OptimalLoad4Gens(active_gens[0], active_gens[1], active_gens[2], active_gens[3]);
		else
			return null;
		
	}
	
	/**Optimal load algorithm for 1 generator
	 * 
	 * @param gen1				Generator number of the running generator
	 * @return 					The optimal load set points.
	 */
	public int[] OptimalLoad1Gen(int gen1){
		double tot_OC = -1;
		int[] temp_load = new int[this.nr_of_gen];
		int[] best_load = new int[this.nr_of_gen];
		for(int i=this.low_limit[gen1]; i<(this.up_limit[gen1]+1); i++){
			temp_load[gen1]=i;

			if(isNecessaryProduction(temp_load)){
				if(tot_OC<0){
					best_load[gen1] = i;
					tot_OC = this.SFOC[0][i-1]*i*this.w_max[0];
				}

				if(this.SFOC[0][i-1]*i*this.w_max[0]<tot_OC){
					best_load[gen1] = i;
					tot_OC = this.SFOC[gen1][i-1]*i*this.w_max[0];
				}
				i=(this.up_limit[gen1]+1);
			}
		}
		return best_load;
	}
	
	/**Optimal load algorithm for 2 generators
	 * 
	 * @param gen1				Generator number of the running generator with priority 1
	 * @param gen2				Generator number of the running generator with priority 2
	 * @return 					The optimal load set points.
	 */
	public int[] OptimalLoad2Gens(int gen1, int gen2){
		double tot_OC = -1;
		int[] temp_load = new int[this.nr_of_gen];
		int[] best_load = new int[this.nr_of_gen];
		for(int i=this.low_limit[gen1]; i<(this.up_limit[gen1]+1); i++){
			for(int j=this.low_limit[gen2]; j<(this.up_limit[gen2]+1); j++){
				temp_load[gen1]=i;
				temp_load[gen2]=j;

				if(isNecessaryProduction(temp_load)){
					if(tot_OC<0){
						best_load[gen1] = i;
						best_load[gen2] = j;
						tot_OC = this.SFOC[0][i-1]*i*this.w_max[0]+this.SFOC[1][j-1]*j*this.w_max[1];
					}

					if(this.SFOC[0][i-1]*i*this.w_max[0]+this.SFOC[1][j-1]*j*this.w_max[1]<tot_OC){
						best_load[gen1] = i;
						best_load[gen2] = j;
						tot_OC = this.SFOC[gen1][i-1]*i*this.w_max[0]+this.SFOC[gen2][j-1]*j*this.w_max[1];
					}
					j=(this.up_limit[gen2]+1);
				}
			}
		}
		return best_load;
	}
	
	/**Optimal load algorithm for 3 generators
	 * 
	 * @param gen1				Generator number of the running generator with priority 1
	 * @param gen2				Generator number of the running generator with priority 2
	 * @param gen3				Generator number of the running generator with priority 3
	 * @return 					The optimal load set points.
	 */
	public int[] OptimalLoad3Gens(int gen1, int gen2, int gen3){
		double tot_OC = -1;
		int[] temp_load = new int[this.nr_of_gen];
		int[] best_load = new int[this.nr_of_gen];
		for(int i=this.low_limit[gen1]; i<(this.up_limit[gen1]+1); i++){
			for(int j=this.low_limit[gen2]; j<(this.up_limit[gen2]+1); j++){
				for(int k=this.low_limit[gen3]; k<(this.up_limit[gen3]+1); k++){
					temp_load[gen1]=i;
					temp_load[gen2]=j;
					temp_load[gen3]=k;

					if(isNecessaryProduction(temp_load)){
						double temp_OC = this.SFOC[0][i-1]*i*this.w_max[0]+this.SFOC[1][j-1]*j*this.w_max[1]
								+this.SFOC[gen3][k-1]*k*this.w_max[2];
						
						if(tot_OC<0){
							best_load[gen1] = i;
							best_load[gen2] = j;
							best_load[gen3] = k;
							tot_OC = temp_OC;
						}

						if(temp_OC<tot_OC){
							best_load[gen1] = i;
							best_load[gen2] = j;
							best_load[gen3] = k;
							tot_OC = temp_OC;
						}
						k=(this.up_limit[gen3]+1);
					}
				}
			}
		}
		return best_load;
	}
	
	/**Optimal load algorithm for 4 generators
	 * 
	 * @param gen1				Generator number of the running generator with priority 1
	 * @param gen2				Generator number of the running generator with priority 2
	 * @param gen3				Generator number of the running generator with priority 3
	 * @param gen4				Generator number of the running generator with priority 4
	 * @return 					The optimal load set points.
	 */
	public int[] OptimalLoad4Gens(int gen1, int gen2, int gen3, int gen4){
		double tot_OC = -1;
		int[] temp_load = new int[this.nr_of_gen];
		int[] best_load = new int[this.nr_of_gen];
		for(int i=this.low_limit[gen1]; i<(this.up_limit[gen1]+1); i++){
			for(int j=this.low_limit[gen2]; j<(this.up_limit[gen2]+1); j++){
				for(int k=this.low_limit[gen3]; k<(this.up_limit[gen3]+1); k++){
					for(int l=this.low_limit[gen4]; l<(this.up_limit[gen4]+1); l++){
						
						temp_load[gen1]=i;
						temp_load[gen2]=j;
						temp_load[gen3]=k;
						temp_load[gen4]=l;

						if(isNecessaryProduction(temp_load)){
							double temp_OC = this.SFOC[0][i-1]*i*this.w_max[0]+this.SFOC[1][j-1]*j*this.w_max[1]
									+this.SFOC[gen3][k-1]*k*this.w_max[2]+this.SFOC[gen4][l-1]*l*this.w_max[3];
							
							if(tot_OC<0){
								best_load[gen1] = i;
								best_load[gen2] = j;
								best_load[gen3] = k;
								best_load[gen4] = l;
								tot_OC = temp_OC;
							}else if(temp_OC<tot_OC){
								best_load[gen1] = i;
								best_load[gen2] = j;
								best_load[gen3] = k;
								best_load[gen4] = l;
								tot_OC = temp_OC;
							}
							l=(this.up_limit[gen4]+1);
						}
					}
				}
			}
		}
		return best_load;
	}

}
