/**This class contains methods for setting the load load set points of the generators in a 
 * unbalanced setting. This is the class called by the unbalLoadOpt class which is the 
 * interface to the prog2 java block.
 * 
 * @author Andreas Carlsen
 *
 */
public class unbalancedLoadSetPoints {

	private double[] low_limit;
	private double[] high_limit;
	private int nr_of_gen;
	private double[] w_max;
	private SFOCVector SFOC;
	private double demand;
	private int[] bin_var;
	private int active_gens = 0;
	private int[] gen_pris;
	
	/**Constructor that initializes the variables
	 * 
	 * @param w_max				Contains the maximum power for each generator
	 * @param SFOC				Contains the SFOC for each generator
	 */
	public unbalancedLoadSetPoints(double[] w_max, SFOCVector SFOC){
		this.nr_of_gen = w_max.length;
		this.w_max = w_max;
		this.SFOC = SFOC;
		this.bin_var = new int[this.nr_of_gen];
		this.gen_pris = new int[this.nr_of_gen];
		this.high_limit = new double[this.nr_of_gen];
		this.low_limit = new double[this.nr_of_gen];
	}

	/**Returns the maximum power for each generator
	 * 
	 * @return					The maximum power for each generator (w_max)
	 */
	public double[] getW_max() {
		return w_max;
	}

	/**Sets the demand
	 * 
	 * @param demand			Power demand
	 */
	public void setDemand(double demand) {
		this.demand = demand;
	}
	
	/**Sets the running bit for each generator
	 * 
	 * @param bin_var			Running bit for each generator
	 */
	public void setBin_var(int[] bin_var) {
		this.active_gens = 0;
		this.bin_var = bin_var;
		for(int i=0; i<bin_var.length; i++){
			if(bin_var[i]==1)
				this.active_gens++;
		}
	}
	
	/**Sets the generator priorities
	 * 
	 * @param gen_pris			Priorities of the generators
	 */
	public void setGen_pris(int[] gen_pris) {
		this.gen_pris = gen_pris;
	}
	
	/**Sets the low limit for each generator
	 * 
	 * @param low_limit			Low limit for each generator
	 */
	public void setLow_limit(double[] low_limit) {
		this.low_limit = low_limit;
	}

	/**Sets the high limit for each generator
	 * 
	 * @param high_limit			High limit for each generator
	 */
	public void setHigh_limit(double[] high_limit) {
		this.high_limit = high_limit;
	}
	
	/**Returns the necessary load of the last generator. Returns -1 if the generator 
	 * can't handle the remaining load.
	 * 
	 * @param temp_load			The load of the other generators.
	 * @param gen				The number representing the generator that will take the remaining load.
	 * @return					The necessary load of the last generator.
	 */
	public double NecessaryProduction(double[] temp_load, int gen){
		double total_production = 0;
		for(int i=0; i<this.nr_of_gen;i++){
			total_production += (temp_load[i]*this.w_max[i]*this.bin_var[i])/100;
		}
		double the_load = ((this.demand-total_production)*100)/(this.w_max[gen]);
		
		if(the_load>=this.low_limit[gen] && the_load<=this.high_limit[gen])
			return the_load;
		else
			return -1;
	}
	
	/**Rearranges the order of the generators if a generator with high priority is not 
	 * running for some reason, then runs the optimization algorithm with the new order.
	 * 
	 * @return 					The optimal load set points.
	 */
	public double[] calculateOptimalLoad(){
		int[] dg_order = new int[this.nr_of_gen];
		int last = this.nr_of_gen-1;
		
		//Rearrange the priorities so that generators with high priorities that has a fault gets the lowest priority.
		if(this.bin_var[gen_pris[0]] == 1){
			dg_order[last+1-this.nr_of_gen]=gen_pris[0];
		}else{
			dg_order[last] = gen_pris[0];
			last--;
		}
		
		if(this.bin_var[gen_pris[1]] == 1){
			dg_order[1+last+1-this.nr_of_gen]=gen_pris[1];
		}else{
			dg_order[last] = gen_pris[1];
			last--;
		}
		
		if(this.bin_var[gen_pris[2]] == 1){
			dg_order[2+last+1-this.nr_of_gen]=gen_pris[2];
		}else{
			dg_order[last] = gen_pris[2];
			last--;
		}
		
		if(this.bin_var[gen_pris[3]] == 1){
			dg_order[3+last+1-this.nr_of_gen]=gen_pris[3];
		}else{
			dg_order[last] = gen_pris[3];
			last--;
		}
		
		//Return the optimal set points
		return  OptimalSetPoints(dg_order[0], dg_order[1], dg_order[2], dg_order[3]);
	}

	/**Optimal load algorithm
	 * 
	 * @param gen1				Generator with highest order (generally priority 1)
	 * @param gen2				Generator with order lower than gen1 (generally priority 2)
	 * @param gen3				Generator with order lower than gen2 (generally priority 3)
	 * @param gen4				Generator with order lower than gen3 (generally priority 4)
	 * @return 					The optimal load set points.
	 */
	public double[] OptimalSetPoints(int gen1, int gen2, int gen3, int gen4){
		double tot_OC = -1;
		double[] temp_load = new double[this.nr_of_gen];
		double[] best_load = new double[this.nr_of_gen];
		for(int i=(int) (this.low_limit[gen1]*this.bin_var[gen1]); i<(this.high_limit[gen1]*this.bin_var[gen1]+1); i++){
			for(int j=(int) (this.low_limit[gen1]*this.bin_var[gen2]); j<(this.high_limit[gen2]*this.bin_var[gen2]+1); j++){
				for(int k=(int) (this.low_limit[gen1]*this.bin_var[gen3]); k<(this.high_limit[gen3]*this.bin_var[gen3]+1); k++){
					if(this.active_gens==4){
						temp_load[gen1]=i;
						temp_load[gen2]=j;
						temp_load[gen3]=k;
						temp_load[gen4]=NecessaryProduction(temp_load, gen4)*this.bin_var[gen4];
					}else if(this.active_gens==3){
						temp_load[gen1]=i;
						temp_load[gen2]=j;
						temp_load[gen3]=NecessaryProduction(temp_load, gen3)*this.bin_var[gen3];
						if(temp_load[gen3]!=-1)
							k=(int)(this.high_limit[gen3]*this.bin_var[gen3]+1);
					}else if(this.active_gens==2){
						temp_load[gen1]=i;
						temp_load[gen2]=NecessaryProduction(temp_load, gen2)*this.bin_var[gen2];
						if(temp_load[gen2]!=-1)
							j=(int)(this.high_limit[gen2]*this.bin_var[gen2]+1);
					}else{
						temp_load[gen1]=NecessaryProduction(temp_load, gen1)*this.bin_var[gen1];
						if(temp_load[gen1]!=-1)
							i=(int)(this.high_limit[gen1]*this.bin_var[gen1]+1);
					}
					
					if(temp_load[gen4]!=-1 && temp_load[gen3]!=-1 && temp_load[gen2]!=-1 && temp_load[gen1]!=-1){
						double temp_OC = this.SFOC.getValueOfIndex(temp_load[gen1], gen1)*temp_load[gen1]*this.w_max[gen1]+this.SFOC.getValueOfIndex(temp_load[gen2], gen2)*temp_load[gen2]*this.w_max[gen2]
								+this.SFOC.getValueOfIndex(temp_load[gen3], gen3)*temp_load[gen3]*this.w_max[gen3]+this.SFOC.getValueOfIndex(temp_load[gen4], gen4)*temp_load[gen4]*this.w_max[gen4];
						
						if(tot_OC<0){
							best_load[gen1] = temp_load[gen1];
							best_load[gen2] = temp_load[gen2];
							best_load[gen3] = temp_load[gen3];
							best_load[gen4] = temp_load[gen4];
							tot_OC = temp_OC;
						}else if(temp_OC<tot_OC){
							best_load[gen1] = temp_load[gen1];
							best_load[gen2] = temp_load[gen2];
							best_load[gen3] = temp_load[gen3];
							best_load[gen4] = temp_load[gen4];
							tot_OC = temp_OC;
						}
					}
					temp_load[gen1]=0;
					temp_load[gen2]=0;
					temp_load[gen3]=0;
					temp_load[gen4]=0;
				}
			}
		}
		return best_load;
	}
	
}
